import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="AuthSession")


@attr.s(auto_attribs=True)
class AuthSession:
    """ """

    created_at: Union[Unset, datetime.datetime] = UNSET
    email: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    ip_address: Union[Unset, str] = UNSET
    is_valid: Union[Unset, bool] = False
    token: Union[Unset, str] = UNSET
    user_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        email = self.email
        id = self.id
        ip_address = self.ip_address
        is_valid = self.is_valid
        token = self.token
        user_id = self.user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if email is not UNSET:
            field_dict["email"] = email
        if id is not UNSET:
            field_dict["id"] = id
        if ip_address is not UNSET:
            field_dict["ip_address"] = ip_address
        if is_valid is not UNSET:
            field_dict["is_valid"] = is_valid
        if token is not UNSET:
            field_dict["token"] = token
        if user_id is not UNSET:
            field_dict["user_id"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _created_at = d.pop("created_at", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        email = d.pop("email", UNSET)

        id = d.pop("id", UNSET)

        ip_address = d.pop("ip_address", UNSET)

        is_valid = d.pop("is_valid", UNSET)

        token = d.pop("token", UNSET)

        user_id = d.pop("user_id", UNSET)

        auth_session = cls(
            created_at=created_at,
            email=email,
            id=id,
            ip_address=ip_address,
            is_valid=is_valid,
            token=token,
            user_id=user_id,
        )

        auth_session.additional_properties = d
        return auth_session

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ValidFileTypes(str, Enum):
    STEP = "step"
    OBJ = "obj"
    STL = "stl"
    DXF = "dxf"
    DWG = "dwg"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.file_conversion import FileConversion
from ...models.valid_file_types import ValidFileTypes
from ...types import Response


def _get_kwargs(
    source_format: ValidFileTypes,
    output_format: ValidFileTypes,
    *,
    client: AuthenticatedClient,
) -> Dict[str, Any]:
    url = "{}/file/conversion/{sourceFormat}/{outputFormat}".format(
        client.base_url, sourceFormat=source_format, outputFormat=output_format
    )

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, FileConversion]]:
    if response.status_code == 200:
        response_200 = FileConversion.from_dict(response.json())

        return response_200
    if response.status_code == 202:
        response_202 = FileConversion.from_dict(response.json())

        return response_202
    if response.status_code == 400:
        response_400 = None

        return response_400
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 406:
        response_406 = None

        return response_406
    if response.status_code == 500:
        response_500 = None

        return response_500
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, FileConversion]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    source_format: ValidFileTypes,
    output_format: ValidFileTypes,
    *,
    client: AuthenticatedClient,
) -> Response[Union[Any, FileConversion]]:
    kwargs = _get_kwargs(
        source_format=source_format,
        output_format=output_format,
        client=client,
    )

    response = httpx.post(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    source_format: ValidFileTypes,
    output_format: ValidFileTypes,
    *,
    client: AuthenticatedClient,
) -> Optional[Union[Any, FileConversion]]:
    """Convert a CAD file from one format to another. If the file being converted is larger than a certain size it will be performed asynchronously."""

    return sync_detailed(
        source_format=source_format,
        output_format=output_format,
        client=client,
    ).parsed


async def asyncio_detailed(
    source_format: ValidFileTypes,
    output_format: ValidFileTypes,
    *,
    client: AuthenticatedClient,
) -> Response[Union[Any, FileConversion]]:
    kwargs = _get_kwargs(
        source_format=source_format,
        output_format=output_format,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    source_format: ValidFileTypes,
    output_format: ValidFileTypes,
    *,
    client: AuthenticatedClient,
) -> Optional[Union[Any, FileConversion]]:
    """Convert a CAD file from one format to another. If the file being converted is larger than a certain size it will be performed asynchronously."""

    return (
        await asyncio_detailed(
            source_format=source_format,
            output_format=output_format,
            client=client,
        )
    ).parsed



""" Contains all the data models used in inputs/outputs """

from .auth_session import AuthSession
from .environment import Environment
from .error_message import ErrorMessage
from .file_conversion import FileConversion
from .file_conversion_status import FileConversionStatus
from .instance_metadata import InstanceMetadata
from .message import Message
from .valid_file_types import ValidFileTypes



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kittycad',
 'kittycad.api',
 'kittycad.api.file',
 'kittycad.api.meta',
 'kittycad.models']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=20.1.0,<22.0.0',
 'httpx>=0.15.4,<0.21.0',
 'python-dateutil>=2.8.0,<3.0.0']

setup_kwargs = {
    'name': 'kittycad',
    'version': '0.0.3',
    'description': 'A client library for accessing KittyCAD',
    'long_description': '# kittycad\nA client library for accessing KittyCAD\n\n## Usage\nFirst, create an authenticated client:\n\n```python\nfrom kittycad import AuthenticatedClient\n\nclient = AuthenticatedClient(token="your_token")\n```\n\nIf you want to use the environment variable `KITTYCAD_API_TOKEN` to do\nauthentication and not pass one to the client, do the following:\n\n```python\nfrom kittycad import AuthenticatedClientFromEnv\n\nclient = AuthenticatedClientFromEnv()\n```\n\nNow call your endpoint and use your models:\n\n```python\nfrom kittycad.models import AuthSession\nfrom kittycad.api.meta import meta_debug_session\nfrom kittycad.types import Response\n\nsession: AuthSession = meta_debug_session.sync(client=client)\n# or if you need more info (e.g. status_code)\nresponse: Response[AuthSession] = meta_debug_session.sync_detailed(client=client)\n```\n\nOr do the same thing with an async version:\n\n```python\nfrom kittycad.models import AuthSession\nfrom kittycad.api.meta import meta_debug_session\nfrom kittycad.types import Response\n\nsession: AuthSession = await meta_debug_session.asyncio(client=client)\nresponse: Response[AuthSession] = await meta_debug_session.asyncio_detailed(client=client)\n```\n',
    'author': None,
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.environment import Environment
from ..types import UNSET, Unset

T = TypeVar("T", bound="InstanceMetadata")


@attr.s(auto_attribs=True)
class InstanceMetadata:
    """ """

    cpu_platform: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    environment: Union[Unset, Environment] = UNSET
    git_hash: Union[Unset, str] = UNSET
    hostname: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    image: Union[Unset, str] = UNSET
    ip_address: Union[Unset, str] = UNSET
    machine_type: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    zone: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        cpu_platform = self.cpu_platform
        description = self.description
        environment: Union[Unset, str] = UNSET
        if not isinstance(self.environment, Unset):
            environment = self.environment.value

        git_hash = self.git_hash
        hostname = self.hostname
        id = self.id
        image = self.image
        ip_address = self.ip_address
        machine_type = self.machine_type
        name = self.name
        zone = self.zone

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if cpu_platform is not UNSET:
            field_dict["cpu_platform"] = cpu_platform
        if description is not UNSET:
            field_dict["description"] = description
        if environment is not UNSET:
            field_dict["environment"] = environment
        if git_hash is not UNSET:
            field_dict["git_hash"] = git_hash
        if hostname is not UNSET:
            field_dict["hostname"] = hostname
        if id is not UNSET:
            field_dict["id"] = id
        if image is not UNSET:
            field_dict["image"] = image
        if ip_address is not UNSET:
            field_dict["ip_address"] = ip_address
        if machine_type is not UNSET:
            field_dict["machine_type"] = machine_type
        if name is not UNSET:
            field_dict["name"] = name
        if zone is not UNSET:
            field_dict["zone"] = zone

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        cpu_platform = d.pop("cpu_platform", UNSET)

        description = d.pop("description", UNSET)

        _environment = d.pop("environment", UNSET)
        environment: Union[Unset, Environment]
        if isinstance(_environment, Unset):
            environment = UNSET
        else:
            environment = Environment(_environment)

        git_hash = d.pop("git_hash", UNSET)

        hostname = d.pop("hostname", UNSET)

        id = d.pop("id", UNSET)

        image = d.pop("image", UNSET)

        ip_address = d.pop("ip_address", UNSET)

        machine_type = d.pop("machine_type", UNSET)

        name = d.pop("name", UNSET)

        zone = d.pop("zone", UNSET)

        instance_metadata = cls(
            cpu_platform=cpu_platform,
            description=description,
            environment=environment,
            git_hash=git_hash,
            hostname=hostname,
            id=id,
            image=image,
            ip_address=ip_address,
            machine_type=machine_type,
            name=name,
            zone=zone,
        )

        instance_metadata.additional_properties = d
        return instance_metadata

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
